/* ===================================================== */
/* =               ChannelMixer v1.0                   = */
/* =              Gustaf Ullberg 2007                  = */
/* ===================================================== */

#include "windows.h"
#include "avisynth.h"
#include "ChannelMixer.h"



/***************************
 * The following is the implementation 
 * of the defined functions.
 ***************************/

ChannelMixer::ChannelMixer(PClip _child, float inRR, float inRG, float inRB, float inGR, float inGG, float inGB, float inBR, float inBG, float inBB, IScriptEnvironment* env) : GenericVideoFilter(_child) {
	// PClip child;   // Contains the source clip.
	// VideoInfo vi;  // Contains videoinfo on the source clip.

	int i, tmp;

	// Is input RGB?
	if(vi.IsRGB24())
		bpp = 3;
	else if(vi.IsRGB32())
		bpp = 4;
	else
		env->ThrowError("ChannelMixer: input to filter must be RGB24 colourspace");   

	// Build look-up tables
	for(i = 0; i < 256; i++) {
		RR[i] = 0.01 * inRR * (float)i;
		RG[i] = 0.01 * inRG * (float)i;
		RB[i] = 0.01 * inRB * (float)i;

		GR[i] = 0.01 * inGR * (float)i;
		GG[i] = 0.01 * inGG * (float)i;
		GB[i] = 0.01 * inGB * (float)i;

		BR[i] = 0.01 * inBR * (float)i;
		BG[i] = 0.01 * inBG * (float)i;
		BB[i] = 0.01 * inBB * (float)i;
	}
}



PVideoFrame __stdcall ChannelMixer::GetFrame(int n, IScriptEnvironment* env) {

	// Request frame 'n' from the child (source) clip.
	PVideoFrame src = child->GetFrame(n, env);
   
	// Construct a frame based on the information of the current frame contained in the "vi" struct.
	PVideoFrame dst = env->NewVideoFrame(vi);

	// Request a Read pointer from the source frame.
	const unsigned char* srcp = src->GetReadPtr();

	// Request a Write pointer from the newly created destination image.
	unsigned char* dstp = dst->GetWritePtr();

	// Requests pitch (length of a line) of the destination image.
	const int dst_pitch = dst->GetPitch();
  
	// Requests rowsize (number of used bytes in a line.
	const int dst_width = dst->GetRowSize();

	// Requests the height of the destination image.
	const int dst_height = dst->GetHeight();

	const int src_pitch = src->GetPitch();
	const int src_width = src->GetRowSize();
	const int src_height = src->GetHeight();
	
	int w, h;
	int Rin, Gin, Bin;
	int Rout, Gout, Bout;

	for (h=0; h < src_height;h++) { // Loop through lines
		for (w = 0; w < src_width; w += bpp) { // Loop through all pixels in each line

			// Read source pixel values
			Bin = *(srcp + w);
			Gin = *(srcp + w + 1);
			Rin = *(srcp + w + 2);

			// Calculate and limit new pixel values
			Rout = RR[Rin] + RG[Gin] + RB[Bin];
			if(Rout > 255) Rout = 255;
			else if(Rout < 0) Rout = 0;
			Gout = GR[Rin] + GG[Gin] + GB[Bin];
			if(Gout > 255) Gout = 255;
			else if(Gout < 0) Gout = 0;
			Bout = BR[Rin] + BG[Gin] + BB[Bin];
			if(Bout > 255) Bout = 255;
			else if(Bout < 0) Bout = 0;

			// Write to destination
			*(dstp + w) = (char)Bout;
			*(dstp + w + 1) = (char)Gout;
			*(dstp + w + 2) = (char)Rout;
		}
		srcp = srcp + src_pitch; // Add the width of one line (in bytes) to the source image.
		dstp = dstp + dst_pitch; // Add the width of one line (in bytes) to the destination.
	}
	
	// Return the destination image
	return dst;
}


// Function that creates the filter
AVSValue __cdecl Create_ChannelMixer(AVSValue args, void* user_data, IScriptEnvironment* env) {
	// Calls the constructor with the arguments provied.
	return new ChannelMixer(args[0].AsClip(), args[1].AsFloat(), args[2].AsFloat(), args[3].AsFloat(), args[4].AsFloat(), args[5].AsFloat(), args[6].AsFloat(), args[7].AsFloat(), args[8].AsFloat(), args[9].AsFloat(), env);
}


// Register the filter in AviSynth
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) {
	env->AddFunction("ChannelMixer", "c[RR]f[RG]f[RB]f[GR]f[GG]f[GB]f[BR]f[BG]f[BB]f", Create_ChannelMixer, 0);
    
    // c - Video Clip
    // i - Integer number
    // f - Float number
    // s - String
    // b - boolean

    return "'ChannelMixer' ChannelMixer plugin";
    // A freeform name of the plugin.
}
